
#include <stdio.h>

#include "socket.h"
#include "nntpclient.h"

int 
doNNTP (char *host, int port, char *newsgroup,char *mailbox)
{
  int socket;
  int rc;
  char buf[1024];
  int first, last, message, localfirst;
  int mboxfd;

  if (verbose)
    printf ("Connecting to %s:%u...\n", host, port);

  /* open the socket and get the greeting */
  if ((socket = Socket (host, port)) < 0)
    {
      perror ("doNNTP: socket");
      return -1;
    }

  if (verbose)
    printf ("Connection established\n");

  rc = NNTP_OK (socket);
  if (rc != 0)
    {
      NNTP_QUIT (socket);
      close (socket);
      return -1;
    }

  rc = NNTP_mode (socket);
  if (rc != 0)
    {
      NNTP_QUIT (socket);
      close (socket);
      return -1;
    }

  rc = NNTP_group (socket, newsgroup, &first, &last);
  if (rc != 0)
    {
      NNTP_QUIT (socket);
      close (socket);
      return -1;
    }

  localfirst = getlastmessage(newsrc,newsgroup);
  if (verbose)
     printf ("Last retreived message is %u\n", localfirst);
  if (localfirst > first)
    first = localfirst;

  for (message = first; message <= last; message++)
    {
      if (verbose)
	printf ("Retreiving message %u of %u...\n", message, last);

      rc = NNTP_article (socket, message);
      if (rc < 0)
	{
	  NNTP_QUIT (socket);
	  close (socket);
	  return -1;
	}
    
      if (rc == 0) {
      mboxfd = openfolder(mailbox);
      readmsg (socket, mboxfd, 0);
      close(mboxfd);
      setlastmessage(newsrc,newsgroup,message);
      }
    }

  NNTP_QUIT (socket);
  return 0;
}

int 
NNTP_OK (int socket)
{
  char buf[1024];

  if (SockGets (socket, buf, sizeof (buf)) == 0)
    {
      if (verbose)
	printf ("< %s\n", buf);

      if (!memcmp (buf, "200", 3))
	return 0;
    }
  return -1;
}

int 
NNTP_QUIT (int socket)
{
  char buf[1024];

  SockPuts (socket, "QUIT");
  if (verbose)
    printf ("> QUIT\n");

  return 0;
}

int 
NNTP_mode (int socket)
{
  char buf[1024];

  SockPuts (socket, "mode reader");
  if (verbose)
    printf ("> mode reader\n");

  if (SockGets (socket, buf, sizeof (buf)) == 0)
    {
      if (verbose)
	printf ("< %s\n", buf);

      if (!memcmp (buf, "200", 3))
	return 0;
    }
  return -1;
}


int 
NNTP_group (int socket, char *newsgroup, int *first, int *last)
{
  char buf[1024];
  int number;

  SockPrintf (socket, "group %s\r\n", newsgroup);
  if (verbose)
    printf ("> group %s\n", newsgroup);

  if (SockGets (socket, buf, sizeof (buf)) != 0)
    return -1;
  if (verbose)
    printf ("< %s\n", buf);

  if (memcmp (buf, "211", 3))
    return -1;
  sscanf (buf + 4, "%d %d %d", &number, first, last);

  if (verbose)
    printf ("Group stat: %u articles, first %u, last %u\n", number, *first, *last);

  return 0;
}


int 
NNTP_article (int socket, int number)
{
  char buf[1024];

  SockPrintf (socket, "article %u\r\n", number);
  if (verbose)
    printf ("> article %u\n", number);

  if (SockGets (socket, buf, sizeof (buf)) == 0)
    {
      if (verbose)
	printf ("< %s\n", buf);

      if (!memcmp (buf, "220", 3))
	return 0;
      if (!memcmp (buf, "423", 3)) {
        fprintf(stderr,"Unable to get article %u\n",number);
	return 1;
      }
    }
  return -1;
}
